Package: kSamples
Type: Package
Title: K-Sample Rank Tests and their Combinations
Version: 1.2-12
Date: 2025-08-25
Authors@R: c(person("Fritz", "Scholz",role=c("aut","cre"),email =
        "fscholz@u.washington.edu"), person("Angie", "Zhu", role=c("aut"),
        email= "a.zhu.stat@gmail.com"))
Author: Fritz Scholz [aut, cre],
  Angie Zhu [aut]
Maintainer: Fritz Scholz <fscholz@u.washington.edu>
Depends: SuppDists
Imports: methods, graphics, stats
Description: Compares k samples using the Anderson-Darling test, Kruskal-Wallis type tests 
	with different rank score criteria, Steel's multiple comparison test, and the 
        Jonckheere-Terpstra (JT) test. It computes asymptotic, simulated or (limited) exact 
        P-values, all valid under randomization, with or without ties, or conditionally 
        under random sampling from populations, given the observed tie pattern.  Except for 
        Steel's test and the JT test it also combines these tests across several blocks of 
	samples.  Also analyzed are 2 x t contingency tables and their blocked combinations 
	using the Kruskal-Wallis criterion.  Steel's test is inverted to provide simultaneous 
	confidence bounds for shift parameters.  A plotting function compares tail probabilities
	obtained under asymptotic approximation with those obtained via simulation or exact 
	calculations.
License: GPL (>= 2)
LazyLoad: yes
NeedsCompilation: yes
Packaged: 2025-08-26 19:04:09 UTC; fritz
Repository: CRAN
Date/Publication: 2025-08-26 23:20:02 UTC
